@@ -1,5 +1,5 @@
-use futures::future::ok;
 use ntex::service::{fn_factory_with_config, fn_service};
+use ntex::util::Ready;
 use ntex_mqtt::v5::codec::PublishAckReason;
 use ntex_mqtt::{v3, v5, MqttServer};
 
@@ -26,9 +26,9 @@ impl std::convert::TryFrom<MyServerError> for v5::PublishAck {
     }
 }
 
-async fn handshake_v3<Io>(
-    handshake: v3::Handshake<Io>,
-) -> Result<v3::HandshakeAck<Io, MySession>, MyServerError> {
+async fn handshake_v3(
+    handshake: v3::Handshake,
+) -> Result<v3::HandshakeAck<MySession>, MyServerError> {
     log::info!("new connection: {:?}", handshake);
 
     let session = MySession { client_id: handshake.packet().client_id.to_string() };
@@ -56,9 +56,9 @@ async fn publish_v3(
     }
 }
 
-async fn handshake_v5<Io>(
-    handshake: v5::Handshake<Io>,
-) -> Result<v5::HandshakeAck<Io, MySession>, MyServerError> {
+async fn handshake_v5(
+    handshake: v5::Handshake,
+) -> Result<v5::HandshakeAck<MySession>, MyServerError> {
     log::info!("new connection: {:?}", handshake);
 
     let session = MySession { client_id: handshake.packet().client_id.to_string() };
@@ -93,18 +93,18 @@ async fn main() -> std::io::Result<()> {
     log::info!("Hello");
 
     ntex::server::Server::build()
-        .bind("mqtt", "127.0.0.1:1883", || {
+        .bind("mqtt", "127.0.0.1:1883", |_| {
             MqttServer::new()
                 .v3(v3::MqttServer::new(handshake_v3).publish(fn_factory_with_config(
                     |session: v3::Session<MySession>| {
-                        ok::<_, MyServerError>(fn_service(move |req| {
+                        Ready::Ok::<_, MyServerError>(fn_service(move |req| {
                             publish_v3(session.clone(), req)
                         }))
                     },
                 )))
                 .v5(v5::MqttServer::new(handshake_v5).publish(fn_factory_with_config(
                     |session: v5::Session<MySession>| {
-                        ok::<_, MyServerError>(fn_service(move |req| {
+                        Ready::Ok::<_, MyServerError>(fn_service(move |req| {
                             publish_v5(session.clone(), req)
                         }))
                     },
